@@ -2,7 +2,7 @@ package Map::Tube::Plugin::FuzzyFind;
 
 use strict;
 use warnings;
-$Map::Tube::Plugin::FuzzyFind::VERSION = '0.02';
+$Map::Tube::Plugin::FuzzyFind::VERSION = '0.03';
 
 =head1 NAME
 
@@ -10,7 +10,7 @@ Map::Tube::Plugin::FuzzyFind - Map::Tube add-on for finding stations and lines b
 
 =head1 VERSION
 
-Version 0.02
+Version 0.03
 
 =cut
 
@@ -126,11 +126,6 @@ The given pattern is matched as a regex (case-insensitively) against the
 line or station names. The pattern may also be specified as a precompiled
 regex. In this case, its case sensitivity will be used unaltered.
 
-=item 'levenshtein'
-
-The closest names as calculated by the Levenshtein edit distance
-(see L<Text::Levenshtein>) will be returned.
-
 =item 'soundex'
 
 All names matching according to the soundex algorithm
@@ -144,6 +139,17 @@ All names matching according to the Metaphone algorithm
 (see L<Text::Metaphone>) will be returned. This is a method that strives to be
 "a modern version of soundex". It is also tuned towards English words.
 
+=item 'levenshtein'
+
+The closest names as calculated by the Levenshtein edit distance
+(see L<Text::Levenshtein>) will be returned.
+
+=item 'ngram'
+
+The closest names as calculated by a comaprison of trigrams
+(see L<String::Trigram>) will be returned. (Future versions may include
+n-grams for n other than 3).
+
 =item 'fuzzy'
 
 Currently, this is a synonym for C<'levenshtein'>.
@@ -185,15 +191,17 @@ no limit will be applied. In scalar context, this parameter is disregarded.
 sub fuzzy_find {  ## no critic(Subroutines::ProhibitExcessComplexity)
   my( $self, @tmpargs ) = @_;
   my( %args, $pattern );
+  # *** This needs to be re-organised into a hash with pointers to methods.
   my %methods = ( exact       => 1,
                   start       => 2,
                   in          => 3,
                   regex       => 4,
                   re          => 4,
-                  levenshtein => 5,
-                  fuzzy       => 5,
-                  soundex     => 6,
-                  metaphone   => 7,
+                  soundex     => 5,
+                  metaphone   => 6,
+                  levenshtein => 7,
+                  fuzzy       => 7,
+                  ngrams      => 8,
                 );
 
   # unify the different ways of passing arguments:
@@ -230,7 +238,6 @@ sub fuzzy_find {  ## no critic(Subroutines::ProhibitExcessComplexity)
   # Now all args are in the non-empty %args hash, and we know what to search for.
   $args{objects} ||= '';
   $args{objects}   = 'stations' if ( $args{objects} eq 'nodes' );
-  $args{maxdist} ||= ( length($pattern) + 1 ) / 2;
   $args{maxsize} ||= 0;
   $args{method}  ||= ( ref( $args{search} ) eq 'Regexp' ) ? 'regex' : 'exact';
 
@@ -268,50 +275,77 @@ sub fuzzy_find {  ## no critic(Subroutines::ProhibitExcessComplexity)
     # Match regex
     $pattern = qr/$pattern/i unless ref($pattern) eq 'Regexp';
     @result  = sort { $a->name() cmp $b->name() } grep { $_->name() =~ $pattern } @{ $source };
-  } elsif ( ( $nmethod >= 5 ) && ( $nmethod <= 7 ) ) {
+  } elsif ( ( $nmethod >= 5 ) && ( $nmethod <= 8 ) ) {
 
     # Use some well-known fuzzy matcher
+    # *** This needs to be cleaned up.
     my( $module, $loaded, $matcher );
     $pattern = lc($pattern);
+    $pattern =~ s/\s+//g;
     try {
       if ( $nmethod == 5 ) {
 
-        # Levenshtein edit distance
-        $module = 'Text::Levenshtein';
-        $loaded = eval qq{ require $module; 1 };
-        croak unless $loaded;
-        $matcher = sub { Text::Levenshtein::distance( $_[0], $pattern ) };
-      } elsif ( $nmethod == 6 ) {
-
         # Knuth's soundex, or some variant thereof, adapted for non-ASCII codes.
         $module = 'Text::Soundex';
         $loaded = eval qq{ require $module; 1 };
         croak unless $loaded;
         my $pattern_soundex = Text::Soundex::soundex_unicode($pattern);
         $matcher = sub {
-                         return ( Text::Soundex::soundex_unicode(@_) eq $pattern_soundex ) ? 0 : 1;
+                         return ( Text::Soundex::soundex_unicode($_[0]) eq $pattern_soundex ) ? 0 : 1;
                        };
         $args{maxdist} = 0.5; # so that exactly the matches will be retained in the map - sort - grep.
-      } elsif ( $nmethod == 7 ) {
+      } elsif ( $nmethod == 6 ) {
 
-        # Metaphone, (claiming to be) "A modern Soundex".
+        # Metaphone, (claiming to be) "A modern Soundex". Still with a slant towards English pronunciation.
         $module = 'Text::Metaphone';
         $loaded = eval qq{ require $module; 1 };
         croak unless $loaded;
         my $pattern_metacode = Text::Metaphone::Metaphone($pattern);
         $matcher = sub {
-                         return ( Text::Metaphone::Metaphone(@_) eq $pattern_metacode ) ? 0 : 1;
+                         return ( Text::Metaphone::Metaphone($_[0]) eq $pattern_metacode ) ? 0 : 1;
                        };
         $args{maxdist} = 0.5; # so that exactly the matches will be retained in the map - sort - grep.
+      } elsif ( $nmethod == 7 ) {
+
+        # Levenshtein edit distance
+        $module = 'Text::Levenshtein';
+        $loaded = eval qq{ require $module; 1 };
+        croak unless $loaded;
+        $args{maxdist} ||= ( length($pattern) + 1 ) / 2;
+        $matcher = sub { Text::Levenshtein::distance( $_[0], $pattern ) };
+      } elsif ( $nmethod == 8 ) {
+
+        # Comparison by number of matching n-grams.
+        # This needs to be re-organised so that multiple calls with the same parameters
+        # (but different search strings) will re-use the string base (not the search string).
+        $module = 'String::Trigram';
+        $loaded = eval qq{ require $module; 1 };
+        croak unless $loaded;
+        $args{maxdist} ||= length($pattern)/2;
+        my %st_args;
+        $st_args{minSim}     = 1 - $args{maxdist}/length($pattern);
+        $st_args{minSim}     = 0 if ( $st_args{minSim} < 0 );
+        $st_args{minSim}     = 1 if ( $st_args{minSim} > 1 );
+        $st_args{ngram}      = $args{windowsize} || 3;
+        $st_args{warp}       = 2;
+        $st_args{cmpBase}    = [ $pattern ];
+        my $trigrammer = String::Trigram->new(%st_args);
+        $matcher = sub {
+                         my $str = $_[0];
+                         $str =~ s/\s+//g;
+                         my %result;
+                         my $nresults = $trigrammer->getSimilarStrings( $str, \%result );
+                         return $nresults ? ( (1 - $result{$pattern}) * length($pattern) ) : ( length($pattern) + 1 );
+                       };
       }
     }
     catch {
-      carp "Matcher module $module not found";
+      croak "Matcher module $module not found";
     };
 
     # $matcher is now a coderef that takes one argument, viz., a string to match.
     # It returns the distance from the pattern; the bigger the distance, the more dissimilar.
-    # In case of matchers that provide just a "macth/no match" information,
+    # In case of matchers that provide just a "match/no match" information,
     # 0 means "match" and 1 means "no match".
 
     my %name2obj = map  { $_->name() => $_ } @{ $source };
